##
# File: RepoHoldingsRemoteDataPrep.py
# Author:  J. Westbrook
# Date:  12-Jul-2018
#
# Note:  This module uses well sandbox and repository defined file names
#        within the configuration defined RCSB exchange sandbox path.
#
# Update:
# 16-Jul-2018 jdw adjust naming to current sandbox conventions.
# 30-Oct-2018 jdw naming to conform to dictioanry and schema conventions.
#  8-Oct-2018 jdw adjustments for greater schema compliance and uniformity
#                 provide alternative date assignment for JSON and mongo validation
# 10-Oct-2018 jdw Add getHoldingsTransferred() for homology models transfered to Model Archive
# 28-Oct-2018 jdw update with semi-colon separated author lists in status and theoretical model files
# 25-Nov-2018 jdw add sequence/pdb_seq_prerelease.fasta and  _rcsb_repository_holdings_prerelease.seq_one_letter_code
# 27-Nov-2018 jdw update rcsb_repository_holdings_current for all entry content types
# 29-Nov-2018 jdw Add support for NMR restraint versions.
# 30-Nov-3018 jdw explicitly filter obsolete entries from current holdings
# 13-Dec-2018 jdw Adjust logic for reporting assembly format availibility
#  5-Feb-2020 jdw Drop superseded entries from the removed entry candidate list.
#                 Avoid overlap between current and removed/unreleased entries.
# 30-Apr-2020 jdw new NMR content types and support for config option RCSB_EDMAP_LIST_PATH
# 23-Oct-2020 jdw add getHoldingsCombined()
# 21-Sep-2021 jdw overhaul using new resource files and with support for remote access
#  4-Feb-2022 dwp Further overhaul for using new resource files and support for remote access
##

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"


import logging


from rcsb.utils.repository.CurrentHoldingsProvider import CurrentHoldingsProvider
from rcsb.utils.repository.RemovedHoldingsProvider import RemovedHoldingsProvider
from rcsb.utils.repository.UnreleasedHoldingsProvider import UnreleasedHoldingsProvider
from rcsb.utils.repository.UpdateHoldingsProvider import UpdateHoldingsProvider


logger = logging.getLogger(__name__)


class RepoHoldingsRemoteDataPrep(object):
    """Assemble repository current and update content holdings."""

    def __init__(self, cachePath=None, useCache=False, **kwargs):
        # filterType is passed along in kwargs to these supporting classes -
        # self.__filterType = kwargs.get("filterType", "")
        # self.__assignDates = "assign-dates" in self.__filterType
        #
        self.__cfgOb = kwargs.get("cfgOb", None)
        #
        self.__chP = CurrentHoldingsProvider(cachePath, useCache, **kwargs)
        self.__uphP = UpdateHoldingsProvider(cachePath, useCache, **kwargs)
        self.__unhP = UnreleasedHoldingsProvider(cachePath, useCache, **kwargs)
        self.__rmhP = RemovedHoldingsProvider(cachePath, useCache, **kwargs)
        self.__currentCacheD = None

    def getHoldingsUpdateEntry(self, updateId):
        dList = []
        retD = self.__uphP.getUpdateData()
        for entryId, qD in retD.items():
            tD = {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId}
            rD = {
                "rcsb_id": entryId,
                "rcsb_repository_holdings_update_entry_container_identifiers": tD,
                "rcsb_repository_holdings_update": qD,
            }
            dList.append(rD)
        return dList

    def getHoldingsCurrentEntry(self, updateId):
        dList = []
        retD = self.__currentCacheD if self.__currentCacheD else self.__getHoldingsCurrent()
        self.__currentCacheD = retD
        for entryId, qD in retD.items():
            tD = (
                {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId, "assembly_ids": qD["assembly_ids"]}
                if "assembly_ids" in qD
                else {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId}
            )
            rD = {
                "rcsb_id": entryId,
                "rcsb_repository_holdings_current_entry_container_identifiers": tD,
                "rcsb_repository_holdings_current": {"repository_content_types": qD["repository_content_types"]},
            }
            dList.append(rD)
        return dList

    def __getHoldingsCurrent(self):
        retD = {}
        rD, assemD = self.__chP.getRcsbContentAndAssemblies()
        for entryId in rD:
            if entryId in assemD:
                retD[entryId] = {"assembly_ids": assemD[entryId], "repository_content_types": rD[entryId]}
            else:
                retD[entryId] = {"repository_content_types": rD[entryId]}
        return retD

    def getHoldingsUnreleasedEntry(self, updateId):
        dList = []

        retD, prD = self.__unhP.getRcsbUnreleasedData()
        #
        currentD = self.__currentCacheD if self.__currentCacheD else self.__getHoldingsCurrent()
        self.__currentCacheD = currentD
        for entryId, qD in retD.items():
            if entryId in currentD:
                continue
            rD = {"rcsb_id": entryId}
            rD["rcsb_repository_holdings_unreleased_entry_container_identifiers"] = {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId}
            if entryId in prD:
                rD["rcsb_repository_holdings_prerelease"] = prD[entryId]
            rD["rcsb_repository_holdings_unreleased"] = qD
            #
            dList.append(rD)
        return dList

    def getHoldingsRemovedEntry(self, updateId):
        dList = []
        trfD, insD, aaD, rmvD, spsD = self.__rmhP.getRcsbRemovedData()
        #
        # rmvD, aaD, spsD = self.__getHoldingsRemoved(dirPath=dirPath)
        # trfD, insD = self.__getHoldingsTransferred(dirPath=dirPath)
        #
        currentD = self.__currentCacheD if self.__currentCacheD else self.__getHoldingsCurrent()
        self.__currentCacheD = currentD
        #
        # Get the list of candidate keys for removed entries -
        #
        entryIdL = sorted(set(list(insD.keys()) + list(rmvD.keys())))
        for entryId in entryIdL:
            if entryId in currentD:
                continue
            rD = {"rcsb_id": entryId}
            rD["rcsb_repository_holdings_removed_entry_container_identifiers"] = {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId}
            #
            if entryId in rmvD:
                rD["rcsb_repository_holdings_removed"] = rmvD[entryId]
            if entryId in aaD:
                rD["rcsb_repository_holdings_removed_audit_author"] = aaD[entryId]
            if entryId in spsD:
                rD["rcsb_repository_holdings_superseded"] = spsD[entryId]
            if entryId in trfD:
                rD["rcsb_repository_holdings_transferred"] = trfD[entryId]
            if entryId in insD:
                rD["rcsb_repository_holdings_insilico_models"] = insD[entryId]
            dList.append(rD)
        return dList

    def getHoldingsCombinedEntry(self, updateId):
        dList = []
        retD = self.__chP.getStatusDetails()
        umD = self.__unhP.getStatusDetails(retD)
        rmD = self.__rmhP.getStatusDetails(retD)
        retD.update(umD)
        retD.update(rmD)
        #
        for entryId, qD in retD.items():
            tD = {"rcsb_id": entryId, "entry_id": entryId, "update_id": updateId}
            rD = {
                "rcsb_id": entryId,
                "rcsb_repository_holdings_combined_entry_container_identifiers": tD,
                "rcsb_repository_holdings_combined": qD,
            }
            dList.append(rD)
        return dList

    #



##
# File: SchemaUpdateExec.py
# Date: 11-Nov-2018  jdw
#
#  Execution wrapper  --  for schema production utilities --
#
#  Updates:
#   13-Dec-2018 jdw add Drugbank and I/HM schema options
#    7-Jan-2019 jdw overhaul
#   23-Nov-2021 dwp Add pdbx_comp_model_core
#
##
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import argparse
import logging
import os

from rcsb.db.define.SchemaDefAccess import SchemaDefAccess
from rcsb.db.utils.SchemaProvider import SchemaProvider
from rcsb.utils.config.ConfigUtil import ConfigUtil

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()


def main():
    parser = argparse.ArgumentParser()
    #
    defaultConfigName = "site_info_configuration"
    #
    parser.add_argument("--update_chem_comp_ref", default=False, action="store_true", help="Update schema for Chemical Component reference definitions")
    parser.add_argument("--update_chem_comp_core_ref", default=False, action="store_true", help="Update core schema for Chemical Component reference definitions")
    parser.add_argument("--update_bird_chem_comp_ref", default=False, action="store_true", help="Update schema for Bird Chemical Component reference definitions")
    parser.add_argument("--update_bird_chem_comp_core_ref", default=False, action="store_true", help="Update core schema for Bird Chemical Component reference definitions")

    parser.add_argument("--update_bird_ref", default=False, action="store_true", help="Update schema for Bird reference definitions")
    parser.add_argument("--update_bird_family_ref", default=False, action="store_true", help="Update schema for Bird Family reference definitions")

    parser.add_argument("--update_pdbx", default=False, action="store_true", help="Update schema for PDBx entry data")
    parser.add_argument("--update_pdbx_core", default=False, action="store_true", help="Update schema for PDBx core entry/entity data")
    parser.add_argument("--update_pdbx_comp_model_core", default=False, action="store_true", help="Update schema for PDBx computational model core entry/entity data")
    #
    parser.add_argument("--update_repository_holdings", default=False, action="store_true", help="Update schema for repository holdings")
    parser.add_argument("--update_entity_sequence_clusters", default=False, action="store_true", help="Update schema for entity sequence clusters")
    parser.add_argument("--update_data_exchange", default=False, action="store_true", help="Update schema for data exchange status")
    parser.add_argument("--update_ihm_dev", default=False, action="store_true", help="Update schema for I/HM dev entry data")
    parser.add_argument("--update_drugbank_core", default=False, action="store_true", help="Update DrugBank schema")
    #
    parser.add_argument("--update_config_all", default=False, action="store_true", help="Update using configuration settings (e.g. DATABASE_NAMES_ALL)")
    parser.add_argument("--update_config_deployed", default=False, action="store_true", help="Update using configuration settings (e.g. DATABASE_NAMES_DEPLOYED)")
    parser.add_argument("--update_config_test", default=False, action="store_true", help="Update using configuration settings (e.g. DATABASE_NAMES_TEST)")
    #
    parser.add_argument("--config_path", default=None, help="Path to configuration options file")
    parser.add_argument("--config_name", default=defaultConfigName, help="Configuration section name")
    #
    parser.add_argument("--cache_path", default=None, help="Schema cache directory path")
    parser.add_argument("--encoding_types", default=None, help="Schema encoding (rcsb|json|bson) (comma separated)")
    parser.add_argument("--validation_levels", default=None, help="Schema validation level (full|min) (comma separated)")
    parser.add_argument("--compare_only", default=False, action="store_true", help="Perform comparison with cached schema")
    #
    parser.add_argument("--debug", default=False, action="store_true", help="Turn on verbose logging")
    parser.add_argument("--mock", default=False, action="store_true", help="Use MOCK repository configuration for dependencies and testing")
    # parser.add_argument("--working_path", default=None, help="Working/alternative path for temporary and schema files")
    args = parser.parse_args()
    #
    debugFlag = args.debug
    if debugFlag:
        logger.setLevel(logging.DEBUG)
    # ----------------------- - ----------------------- - ----------------------- - ----------------------- - ----------------------- -
    #                                       Configuration Details
    configPath = args.config_path
    configName = args.config_name
    cachePath = args.cache_path
    compareOnly = args.compare_only
    #
    encodingTypes = args.encoding_types.split(",") if args.encoding_types else []
    validationLevels = args.validation_levels.split(",") if args.validation_levels else []
    dataTypingList = ["ANY", "SQL"]

    if not configPath:
        configPath = os.getenv("DBLOAD_CONFIG_PATH", None)
    try:
        if os.access(configPath, os.R_OK):
            os.environ["DBLOAD_CONFIG_PATH"] = configPath
            logger.info("Using configuation path %s (%s)", configPath, configName)
        else:
            logger.error("Missing or access issue with config file %r", configPath)
            exit(1)
        mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data") if args.mock else None
        cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=defaultConfigName, mockTopPath=mockTopPath)
        if configName != defaultConfigName:
            cfgOb.replaceSectionName(defaultConfigName, configName)
    except Exception as e:
        logger.error("Missing or access issue with config file %r with %s", configPath, str(e))
        exit(1)
    #
    databaseNameList = []
    if args.update_chem_comp_ref:
        databaseNameList.append("chem_comp")

    if args.update_bird_chem_comp_ref:
        databaseNameList.append("bird_chem_comp")

    if args.update_chem_comp_core_ref:
        databaseNameList.append("chem_comp_core")

    if args.update_bird_chem_comp_core_ref:
        databaseNameList.append("bird_chem_comp_core")

    if args.update_bird_ref:
        databaseNameList.append("bird")

    if args.update_bird_family_ref:
        databaseNameList.append("bird_family")

    if args.update_pdbx:
        databaseNameList.append("pdbx")

    if args.update_pdbx_core:
        databaseNameList.append("pdbx_core")

    if args.update_pdbx_comp_model_core:
        databaseNameList.append("pdbx_comp_model_core")

    if args.update_repository_holdings:
        databaseNameList.append("repository_holdings")

    if args.update_entity_sequence_clusters:
        databaseNameList.append("sequence_clusters")

    if args.update_data_exchange:
        databaseNameList.append("data_exchange")

    if args.update_ihm_dev:
        databaseNameList.append("ihm_dev")

    if args.update_drugbank_core:
        databaseNameList.append("drugbank_core")

    if args.update_config_deployed:
        databaseNameList = cfgOb.getList("DATABASE_NAMES_DEPLOYED", sectionName="database_catalog_configuration")
        dataTypingList = cfgOb.getList("DATATYPING_DEPLOYED", sectionName="database_catalog_configuration")
        validationLevels = cfgOb.getList("VALIDATION_LEVELS_DEPLOYED", sectionName="database_catalog_configuration")
        encodingTypes = cfgOb.getList("ENCODING_TYPES_DEPLOYED", sectionName="database_catalog_configuration")

    if args.update_config_all:
        databaseNameList = cfgOb.getList("DATABASE_NAMES_ALL", sectionName="database_catalog_configuration")
        dataTypingList = cfgOb.getList("DATATYPING_ALL", sectionName="database_catalog_configuration")
        validationLevels = cfgOb.getList("VALIDATION_LEVELS_ALL", sectionName="database_catalog_configuration")
        encodingTypes = cfgOb.getList("ENCODING_TYPES_ALL", sectionName="database_catalog_configuration")

    if args.update_config_test:
        databaseNameList = cfgOb.getList("DATABASE_NAMES_TEST", sectionName="database_catalog_configuration")
        dataTypingList = cfgOb.getList("DATATYPING_TEST", sectionName="database_catalog_configuration")
        validationLevels = cfgOb.getList("VALIDATION_LEVELS_TEST", sectionName="database_catalog_configuration")
        encodingTypes = cfgOb.getList("ENCODING_TYPES_TEST", sectionName="database_catalog_configuration")
    #
    scnD = cfgOb.get("document_collection_names", sectionName="document_helper_configuration")
    #
    databaseNameList = list(set(databaseNameList))
    logger.debug("Collections %s", list(scnD.items()))
    logger.debug("databaseNameList %s", databaseNameList)

    if compareOnly:
        schP = SchemaProvider(cfgOb, cachePath, useCache=True)
        difPathList = []
        for databaseName in databaseNameList:
            for dataTyping in dataTypingList:
                logger.debug("Building schema %s with types %s", databaseName, dataTyping)
                pth = schP.schemaDefCompare(databaseName, dataTyping)
                if pth:
                    difPathList.append(pth)
        if difPathList:
            logger.info("Schema definition difference path list %r", difPathList)
        difPathList = []
        for databaseName in databaseNameList:
            dD = schP.makeSchemaDef(databaseName, dataTyping="ANY", saveSchema=False)
            sD = SchemaDefAccess(dD)
            for cd in sD.getCollectionInfo():
                collectionName = cd["NAME"]
                for encodingType in encodingTypes:
                    if encodingType.lower() != "json":
                        continue
                    for level in validationLevels:
                        pth = schP.jsonSchemaCompare(databaseName, collectionName, encodingType, level)
                        if pth:
                            difPathList.append(pth)
        if difPathList:
            logger.info("JSON schema difference path list %r", difPathList)

    else:
        schP = SchemaProvider(cfgOb, cachePath, useCache=False)
        for databaseName in databaseNameList:
            for encodingType in encodingTypes:
                if encodingType == "rcsb":
                    for dataTyping in dataTypingList:
                        logger.info("Creating schema definition for content type %s data typing %s", databaseName, dataTyping)
                        schP.makeSchemaDef(databaseName, dataTyping=dataTyping, saveSchema=True)
                else:
                    if databaseName in scnD:
                        for dD in scnD[databaseName]:
                            collectionName = dD["NAME"]
                            for validationLevel in validationLevels:
                                logger.info("Creating %r schema for content type %s collection %s", encodingType, databaseName, collectionName)
                                schP.makeSchema(databaseName, collectionName, encodingType=encodingType, level=validationLevel, saveSchema=True)


if __name__ == "__main__":
    main()



##
# File:    CrateDbUtil.py
# Author:  J. Westbrook
# Date:    21-Dec-2017
# Version: 0.001 Initial version
#
# Updates:
#
##
"""
Utility classes to create connections and process SQL commands with CrateDb.

"""
from __future__ import generators

import logging

from crate.client.exceptions import DatabaseError, OperationalError, ProgrammingError, Warning  # pylint: disable=redefined-builtin

__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

logger = logging.getLogger(__name__)


class CrateDbQuery(object):

    """Parameterized SQL queries using Python DBI protocol..."""

    def __init__(self, dbcon, verbose=True):
        self.__dbcon = dbcon
        self.__verbose = verbose
        self.__ops = ["EQ", "GE", "GT", "LT", "LE", "LIKE", "NOT LIKE"]
        self.__opDict = {"EQ": "=", "GE": ">=", "GT": ">", "LT": "<", "LE": "<=", "LIKE": "LIKE", "NOT LIKE": "NOT LIKE"}
        self.__logOps = ["AND", "OR", "NOT"]
        self.__grpOps = ["BEGIN", "END"]
        self.__warningAction = "default"

    def sqlTemplateCommandMany(self, sqlTemplate, valueLists=None):
        """Execute a batch sql commands followed by a single commit. Commands are
         are describe in a template with an associated list of values.

        cursor.executemany("INSERT INTO locations (name, date, kind, position) VALUES (?, ?, ?, ?)",
            ...                [('Cloverleaf', '2007-03-11', 'Quasar', 7),
            ...                 ('Old Faithful', '2007-03-11', 'Quasar', 7)])
            [{u'rowcount': 1}, {u'rowcount': 1}]

        """
        lenR = 0
        ret = []
        iFail = 0
        try:
            lenT = len(valueLists)
            curs = self.__dbcon.cursor()
            ret = curs.executemany(sqlTemplate, valueLists)
            lenR = len(ret)
            logger.debug("Return len %d", len(ret))
            for i, _ in enumerate(ret):
                if ret[i]["rowcount"] != 1:
                    iFail += 1
                    logger.info("Insert fails on row %d of %d with values: %r", i, lenT, valueLists[i])
            curs.close()
            return lenR - iFail
        except DatabaseError as e:
            logger.info("sqlTemplate %s", sqlTemplate)
            logger.info("return list %r", ret)
            logger.error("error is:\n%s", str(e))
            curs.close()
        except Warning as e:
            logger.warning("warning is:\n%s", str(e))
            curs.close()
        except Exception as e:
            logger.exception("Exception is:\n%s", str(e))
            curs.close()
        #
        lenR = len(ret)
        return lenR - iFail

    def sqlTemplateCommand(self, sqlTemplate=None, valueList=None):
        """Execute sql template command with associated value list.

        Insert one row -

        Errors and warnings that generate exceptions are caught by this method.
        """
        valueList = valueList if valueList else []
        try:
            curs = self.__dbcon.cursor()
            curs.execute(sqlTemplate, valueList)
            curs.close()
            return True
        except DatabaseError as e:
            logger.info(" error is:\n%s\n", str(e))
            curs.close()
        except Warning as e:
            logger.info(" warning is:\n%s\n", str(e))
            curs.close()
        except Exception as e:
            logger.info(" exception is:\n%s\n", str(e))
            curs.close()
        return False

    def sqlTemplateCommandList(self, sqlTemplateValueList=None):
        """Execute sql template command with associated value list.

        Input -

        sqlTemplateValueList [(sqlTemplate,vList), (sqlTemplate, vlist), ... ]

        Insert on row -

        Errors and warnings that generate exceptions are caught by this method.
        """
        vL = []
        iFail = 0
        try:
            curs = self.__dbcon.cursor()
            #
            lenT = len(sqlTemplateValueList)
            for ii in range(lenT):
                tV, vL = sqlTemplateValueList[ii]
                try:
                    curs.execute(tV, vL)
                except Exception as e:
                    iFail += 1
                    logger.info(" Error is: %s", str(e))
                    # logger.info(" Template for record %d of %d : %s" % (ii, lenT, t))
                    logger.info(" Record %d of %d value list: %s", ii, lenT, vL)
            #
            curs.close()
            logger.debug(" Inserted %d of %d values", ii - iFail, lenT)
            return ii - iFail + 1
        except DatabaseError as e:
            logger.exception(" error is: %s", str(e))
            logger.info(" Record %d of %d value list: %s", ii, lenT, vL)
            curs.close()
        except Warning as e:
            logger.info(" Warning is: %s", str(e))
            logger.info(" Record %d of %d value list: %s", ii, lenT, vL)
            curs.close()
        except Exception as e:
            logger.info(" Exception is: %s", str(e))
            logger.info(" Record %d of %d value list: %s", ii, lenT, vL)
            curs.close()
        return ii - iFail + 1

    def sqlCommandList(self, sqlCommandList):
        """Execute the input list of SQL commands catching exceptions from the server.

        The treatment of warning is controlled by a prior setting of self.setWarnings("error"|"ignore"|"default")
        """

        try:
            sqlCommand = ""
            curs = self.__dbcon.cursor()
            for sqlCommand in sqlCommandList:
                curs.execute(sqlCommand)
            #
            curs.close()
            return True
        except DatabaseError as e:
            logger.info(" SQL command failed for:\n%s", sqlCommand)
            logger.info(" database error is message is:\n%s", str(e))
            curs.close()
        except Warning as e:
            logger.info(" warning message is:\n%s", str(e))
            logger.info(" generated warnings for command:\n%s", sqlCommand)
            curs.close()
        except Exception as e:
            logger.info(" exception message is:\n%s\n", str(e))
            logger.exception(" SQL command failed for:\n%s\n", sqlCommand)
            curs.close()

        return False

    def sqlCommand(self, queryString):
        """Execute SQL command catching exceptions returning no data from the server."""
        try:
            curs = self.__dbcon.cursor()
            curs.execute(queryString)
            curs.close()
            return True
        except OperationalError as e:
            logger.info(" SQL command failed for:\n%s", queryString)
            logger.info(" warning is message is:\n%s", str(e))
            curs.close()
        except DatabaseError as e:
            logger.info(" SQL command failed for:\n%s\n", queryString)
            logger.info(" MySQL warning is message is:\n%s\n", str(e))
            curs.close()
        except Exception as e:
            logger.exception(" SQL command failed for:\n%s\n with %s", queryString, str(e))
            curs.close()
        return []

    def __fetchIter(self, cursor, rowSize=1000):
        """Chunked iterator to manage results fetches to mysql server"""
        while True:
            results = cursor.fetchmany(rowSize)
            if not results:
                break
            for result in results:
                yield result

    def selectRows(self, queryString):
        """Execute SQL command and return list of lists for the result set."""
        rowList = []
        try:
            curs = self.__dbcon.cursor()
            curs.execute(queryString)
            while True:
                result = curs.fetchone()
                if result is not None:
                    rowList.append(result)
                else:
                    break
            curs.close()
            return rowList
        except ProgrammingError as e:
            logger.info(" MySQL warning is message is:\n%s\n", str(e))
            curs.close()
        except OperationalError as e:
            logger.info(" MySQL warning is message is:\n%s\n", str(e))
            logger.info(" SQL command failed for:\n%s\n", queryString)
            curs.close()
        except DatabaseError as e:
            logger.info(" MySQL warning is message is:\n%s\n", str(e))
            logger.info(" SQL command failed for:\n%s\n", queryString)
            curs.close()
        except Exception as e:
            logger.exception(" SQL command failed for:\n%s\n with %s", queryString, str(e))
            curs.close()

        return []

    def simpleQuery(self, selectList=None, fromList=None, condition="", orderList=None, returnObj=None):
        """ """
        #
        selectList = selectList if selectList else []
        fromList = fromList if fromList else []
        orderList = orderList if orderList else []
        returnObj = returnObj if returnObj else []
        colsCsv = ",".join(["%s" % k for k in selectList])
        tablesCsv = ",".join(["%s" % k for k in fromList])

        order = ""
        if orderList:
            (aV, tV) = orderList[0]
            order = " ORDER BY CAST(%s AS %s) " % (aV, tV)
            for (aV, tV) in orderList[1:]:
                order += ", CAST(%s AS %s) " % (aV, tV)

        #
        query = "SELECT " + colsCsv + " FROM " + tablesCsv + condition + order
        if self.__verbose:
            logger.info("Query: %s\n", query)
        curs = self.__dbcon.cursor()
        curs.execute(query)
        while True:
            result = curs.fetchone()
            if result is not None:
                returnObj.append(result)
            else:
                break
        curs.close()
        return returnObj

    def testSelectQuery(self, count):
        tSQL = "select %d" % count
        #
        try:
            rowL = self.selectRows(queryString=tSQL)
            tup = rowL[0]
            return int(str(tup[0])) == count
        except Exception:
            return False
